@@ -1,3 +1,5 @@
+require "rbconfig"
+
 require 'rake/baseextensiontask'
 
 # Define a series of tasks to aid in the compilation of Java extensions for
@@ -29,8 +31,8 @@ module Rake
       @classpath      = nil
       @java_compiling = nil
       @debug          = false
-      @source_version = '1.5'
-      @target_version = '1.5'
+      @source_version = '1.6'
+      @target_version = '1.6'
     end
 
     def define
@@ -211,11 +213,12 @@ execute the Rake compilation task using the JRuby interpreter.
         end
       end
       unless jruby_cpath
-        jruby_cpath = ENV['JRUBY_PARENT_CLASSPATH'] || ENV['JRUBY_HOME'] &&
-          Dir.glob("#{File.expand_path(ENV['JRUBY_HOME'])}/lib/*.jar").
-            join(File::PATH_SEPARATOR)
+        libdir = RbConfig::CONFIG['libdir']
+        if libdir.start_with? "classpath:"
+          raise 'Cannot build with jruby-complete'
+        end
+        jruby_cpath = File.join(libdir, "jruby.jar")
       end
-      raise "JRUBY_HOME or JRUBY_PARENT_CLASSPATH are not set" unless jruby_cpath
       jruby_cpath += File::PATH_SEPARATOR + args.join(File::PATH_SEPARATOR) unless args.empty?
       jruby_cpath ? "-cp \"#{jruby_cpath}\"" : ""
     end
