@@ -17,6 +17,11 @@ describe Rake::ExtensionTask do
         ext.name.should == 'extension_one'
       end
 
+      it 'should allow symbol as extension name assignation' do
+        ext = Rake::ExtensionTask.new(:extension_one)
+        ext.name.should == 'extension_one'
+      end
+
       it 'should allow string as extension name using block assignation' do
         ext = Rake::ExtensionTask.new do |ext|
           ext.name = 'extension_two'
@@ -72,8 +77,8 @@ describe Rake::ExtensionTask do
       @ext.lib_dir.should == 'lib'
     end
 
-    it 'should look for C files pattern (.c)' do
-      @ext.source_pattern.should == "*.c"
+    it 'should look for C/C++ files pattern (.c,.cc,.cpp)' do
+      @ext.source_pattern.should == "*.{c,cc,cpp}"
     end
 
     it 'should have no configuration options preset to delegate' do
@@ -89,7 +94,7 @@ describe Rake::ExtensionTask do
     end
 
     it 'should default to no cross compilation' do
-      @ext.cross_compile.should be_false
+      @ext.cross_compile.should == false
     end
 
     it 'should have no configuration options for cross compilation' do
@@ -110,7 +115,7 @@ describe Rake::ExtensionTask do
 
     context '(one extension)' do
       before :each do
-        Rake::FileList.stub!(:[]).and_return(["ext/extension_one/source.c"], [])
+        allow(Rake::FileList).to receive(:[]).and_return(["ext/extension_one/source.c"], [])
         @ext = Rake::ExtensionTask.new('extension_one')
         @ext_bin = ext_bin('extension_one')
         @platform = RUBY_PLATFORM
@@ -119,7 +124,7 @@ describe Rake::ExtensionTask do
 
       context 'compile' do
         it 'should define as task' do
-          Rake::Task.task_defined?('compile').should be_true
+          Rake::Task.task_defined?('compile').should == true
         end
 
         it "should depend on 'compile:{platform}'" do
@@ -129,7 +134,7 @@ describe Rake::ExtensionTask do
 
       context 'compile:extension_one' do
         it 'should define as task' do
-          Rake::Task.task_defined?('compile:extension_one').should be_true
+          Rake::Task.task_defined?('compile:extension_one').should == true
         end
 
         it "should depend on 'compile:extension_one:{platform}'" do
@@ -139,7 +144,7 @@ describe Rake::ExtensionTask do
 
       context 'lib/extension_one.{so,bundle}' do
         it 'should define as task' do
-          Rake::Task.task_defined?("lib/#{@ext_bin}").should be_true
+          Rake::Task.task_defined?("lib/#{@ext_bin}").should == true
         end
 
         it "should depend on 'copy:extension_one:{platform}:{ruby_ver}'" do
@@ -149,7 +154,7 @@ describe Rake::ExtensionTask do
 
       context 'tmp/{platform}/extension_one/{ruby_ver}/extension_one.{so,bundle}' do
         it 'should define as task' do
-          Rake::Task.task_defined?("tmp/#{@platform}/extension_one/#{@ruby_ver}/#{@ext_bin}").should be_true
+          Rake::Task.task_defined?("tmp/#{@platform}/extension_one/#{@ruby_ver}/#{@ext_bin}").should == true
         end
 
         it "should depend on 'tmp/{platform}/extension_one/{ruby_ver}/Makefile'" do
@@ -167,7 +172,7 @@ describe Rake::ExtensionTask do
 
       context 'tmp/{platform}/extension_one/{ruby_ver}/Makefile' do
         it 'should define as task' do
-          Rake::Task.task_defined?("tmp/#{@platform}/extension_one/#{@ruby_ver}/Makefile").should be_true
+          Rake::Task.task_defined?("tmp/#{@platform}/extension_one/#{@ruby_ver}/Makefile").should == true
         end
 
         it "should depend on 'tmp/{platform}/extension_one/{ruby_ver}'" do
@@ -196,7 +201,7 @@ describe Rake::ExtensionTask do
       end
 
       it "should warn when pre-compiled files exist in extension directory" do
-        Rake::FileList.stub!(:[]).
+        allow(Rake::FileList).to receive(:[]).
           and_return(["ext/extension_one/source.c"],
                       ["ext/extension_one/source.o"])
 
@@ -209,7 +214,7 @@ describe Rake::ExtensionTask do
 
     context '(extension in custom location)' do
       before :each do
-        Rake::FileList.stub!(:[]).and_return(["ext/extension_one/source.c"], [])
+        allow(Rake::FileList).to receive(:[]).and_return(["ext/extension_one/source.c"], [])
         @ext = Rake::ExtensionTask.new('extension_one') do |ext|
           ext.ext_dir = 'custom/ext/foo'
         end
@@ -227,7 +232,7 @@ describe Rake::ExtensionTask do
 
     context '(native tasks)' do
       before :each do
-        Rake::FileList.stub!(:[]).and_return(["ext/extension_one/source.c"], [])
+        allow(Rake::FileList).to receive(:[]).and_return(["ext/extension_one/source.c"], [])
         @spec = mock_gem_spec
         @ext_bin = ext_bin('extension_one')
         @platform = RUBY_PLATFORM
@@ -236,24 +241,24 @@ describe Rake::ExtensionTask do
 
       context 'native' do
         before :each do
-          @spec.stub!(:platform=).and_return('ruby')
+          allow(@spec).to receive(:platform=).and_return('ruby')
         end
 
         it 'should define a task for building the supplied gem' do
           Rake::ExtensionTask.new('extension_one', @spec)
-          Rake::Task.task_defined?('native:my_gem').should be_true
+          Rake::Task.task_defined?('native:my_gem').should == true
         end
 
         it 'should define as task for pure ruby gems' do
-          Rake::Task.task_defined?('native').should be_false
+          Rake::Task.task_defined?('native').should == false
           Rake::ExtensionTask.new('extension_one', @spec)
-          Rake::Task.task_defined?('native').should be_true
+          Rake::Task.task_defined?('native').should == true
         end
 
         it 'should not define a task for already native gems' do
-          @spec.stub!(:platform).and_return('current')
+          allow(@spec).to receive(:platform).and_return('current')
           Rake::ExtensionTask.new('extension_one', @spec)
-          Rake::Task.task_defined?('native').should be_false
+          Rake::Task.task_defined?('native').should == false
         end
 
         it 'should depend on platform specific native tasks' do
@@ -270,23 +275,81 @@ describe Rake::ExtensionTask do
       end
     end
 
+    context '(one extension whose name with directory prefixes)' do
+      before :each do
+        allow(Rake::FileList).to receive(:[]).and_return(["ext/prefix1/prefix2/extension_one/source.c"], [])
+        @ext = Rake::ExtensionTask.new('prefix1/prefix2/extension_one')
+        @ext_bin = ext_bin('extension_one')
+        @platform = RUBY_PLATFORM
+        @ruby_ver = RUBY_VERSION
+      end
+
+      context 'compile' do
+        it 'should define as task' do
+          Rake::Task.task_defined?('compile').should == true
+        end
+
+        it "should depend on 'compile:{platform}'" do
+          Rake::Task['compile'].prerequisites.should include("compile:#{@platform}")
+        end
+      end
+
+      context 'compile:prefix1/prefix2/extension_one' do
+        it 'should define as task' do
+          Rake::Task.task_defined?('compile:prefix1/prefix2/extension_one').should == true
+        end
+
+        it "should depend on 'compile:prefix1/prefix2/extension_one:{platform}'" do
+          Rake::Task['compile:prefix1/prefix2/extension_one'].prerequisites.should include("compile:prefix1/prefix2/extension_one:#{@platform}")
+        end
+      end
+
+      context 'lib/prefix1/prefix2/extension_one.{so,bundle}' do
+        it 'should define as task' do
+          Rake::Task.task_defined?("lib/prefix1/prefix2/#{@ext_bin}").should == true
+        end
+
+        it "should depend on 'copy:prefix1/prefix2/extension_one:{platform}:{ruby_ver}'" do
+          Rake::Task["lib/prefix1/prefix2/#{@ext_bin}"].prerequisites.should include("copy:prefix1/prefix2/extension_one:#{@platform}:#{@ruby_ver}")
+        end
+      end
+
+      context 'tmp/{platform}/prefix1/prefix2/extension_one/{ruby_ver}/prefix1/prefix2/extension_one.{so,bundle}' do
+        it 'should define as task' do
+          Rake::Task.task_defined?("tmp/#{@platform}/prefix1/prefix2/extension_one/#{@ruby_ver}/prefix1/prefix2/#{@ext_bin}").should == true
+        end
+
+        it "should depend on 'tmp/{platform}/prefix1/prefix2/extension_one/{ruby_ver}/Makefile'" do
+          Rake::Task["tmp/#{@platform}/prefix1/prefix2/extension_one/#{@ruby_ver}/prefix1/prefix2/#{@ext_bin}"].prerequisites.should include("tmp/#{@platform}/prefix1/prefix2/extension_one/#{@ruby_ver}/Makefile")
+        end
+
+        it "should depend on 'ext/extension_one/source.c'" do
+          Rake::Task["tmp/#{@platform}/prefix1/prefix2/extension_one/#{@ruby_ver}/prefix1/prefix2/#{@ext_bin}"].prerequisites.should include("ext/prefix1/prefix2/extension_one/source.c")
+        end
+
+        it "should not depend on 'ext/extension_one/source.h'" do
+          Rake::Task["tmp/#{@platform}/prefix1/prefix2/extension_one/#{@ruby_ver}/prefix1/prefix2/#{@ext_bin}"].prerequisites.should_not include("ext/prefix1/prefix2/extension_one/source.h")
+        end
+      end
+    end
+
     context '(cross platform tasks)' do
       before :each do
-        File.stub!(:exist?).and_return(true)
-        YAML.stub!(:load_file).and_return(mock_config_yml)
-        Rake::FileList.stub!(:[]).and_return(["ext/extension_one/source.c"], [])
+        allow(File).to receive(:exist?).and_return(true)
+        allow(YAML).to receive(:load_file).and_return(mock_config_yml)
+        allow(Rake::FileList).to receive(:[]).and_return(["ext/extension_one/source.c"], [])
         @spec = mock_gem_spec
         @config_file = File.expand_path("~/.rake-compiler/config.yml")
         @ruby_ver = RUBY_VERSION
         @platform = 'i386-mingw32'
         @config_path = mock_config_yml["rbconfig-#{@platform}-#{@ruby_ver}"]
 
-        File.stub!(:open).and_yield(mock_fake_rb)
+        allow(File).to receive(:open).and_yield(mock_fake_rb)
       end
 
       context 'if no rake-compiler configuration exists' do
         before :each do
-          File.should_receive(:exist?).with(@config_file).and_return(false)
+          expect(File).to receive(:exist?).with(@config_file).and_return(false)
 
           _, @err = capture_output do
             Rake::ExtensionTask.new('extension_one') do |ext|
@@ -310,16 +373,16 @@ describe Rake::ExtensionTask do
       end
 
       it 'should parse the config file using YAML' do
-        YAML.should_receive(:load_file).with(@config_file).and_return(mock_config_yml)
+        expect(YAML).to receive(:load_file).with(@config_file).and_return(mock_config_yml)
         Rake::ExtensionTask.new('extension_one') do |ext|
           ext.cross_compile = true
         end
       end
 
       it 'should warn if no section of config file defines running version of ruby' do
-        config = mock(Hash)
-        config.should_receive(:[]).with("rbconfig-#{@platform}-#{@ruby_ver}").and_return(nil)
-        YAML.stub!(:load_file).and_return(config)
+        config = Hash.new
+        expect(config).to receive(:[]).with("rbconfig-#{@platform}-#{@ruby_ver}").and_return(nil)
+        allow(YAML).to receive(:load_file).and_return(config)
         out, err = capture_output do
           Rake::ExtensionTask.new('extension_one') do |ext|
             ext.cross_compile = true
@@ -339,9 +402,9 @@ describe Rake::ExtensionTask do
       end
 
       it 'should generate additional rake tasks if files are added when cross compiling' do
-        config = mock(Hash)
-        config.stub!(:[]).and_return('/rubies/1.9.1/rbconfig.rb')
-        YAML.stub!(:load_file).and_return(config)
+        config = Hash.new
+        allow(config).to receive(:[]).and_return('/rubies/1.9.1/rbconfig.rb')
+        allow(YAML).to receive(:load_file).and_return(config)
 
         # Use a real spec instead of a mock because define_native_tasks dups and
         # calls methods on Gem::Specification, which is more than mock can do.
@@ -353,7 +416,7 @@ describe Rake::ExtensionTask do
         # Gem::PackageTask calls Rake::PackageTask which sets Gem.configuration.verbose,
         # which initializes Gem::ConfigFile,
         # which gets mad if it cannot find `sysconfdir`/gemrc
-        Gem.stub_chain(:configuration, :verbose=).and_return(true)
+        allow(Gem).to receive_message_chain(:configuration, :verbose=).and_return(true)
 
         ENV['RUBY_CC_VERSION'] = '1.9.1'
         Rake::ExtensionTask.new('extension_one', spec) do |ext|
@@ -369,9 +432,9 @@ describe Rake::ExtensionTask do
       end
 
       it 'should allow usage of RUBY_CC_VERSION to indicate a different version of ruby' do
-        config = mock(Hash)
-        config.should_receive(:[]).with("rbconfig-i386-mingw32-1.9.1").and_return('/rubies/1.9.1/rbconfig.rb')
-        YAML.stub!(:load_file).and_return(config)
+        config = Hash.new
+        expect(config).to receive(:[]).with("rbconfig-i386-mingw32-1.9.1").and_return('/rubies/1.9.1/rbconfig.rb')
+        allow(YAML).to receive(:load_file).and_return(config)
 
         ENV['RUBY_CC_VERSION'] = '1.9.1'
         Rake::ExtensionTask.new('extension_one') do |ext|
@@ -380,10 +443,10 @@ describe Rake::ExtensionTask do
       end
 
       it 'should allow multiple versions be supplied to RUBY_CC_VERSION' do
-        config = mock(Hash)
-        config.should_receive(:[]).once.with("rbconfig-i386-mingw32-1.8.6").and_return('/rubies/1.8.6/rbconfig.rb')
-        config.should_receive(:[]).once.with("rbconfig-i386-mingw32-1.9.1").and_return('/rubies/1.9.1/rbconfig.rb')
-        YAML.stub!(:load_file).and_return(config)
+        config = Hash.new
+        expect(config).to receive(:[]).once.with("rbconfig-i386-mingw32-1.8.6").and_return('/rubies/1.8.6/rbconfig.rb')
+        expect(config).to receive(:[]).once.with("rbconfig-i386-mingw32-1.9.1").and_return('/rubies/1.9.1/rbconfig.rb')
+        allow(YAML).to receive(:load_file).and_return(config)
 
         ENV['RUBY_CC_VERSION'] = '1.8.6:1.9.1'
         Rake::ExtensionTask.new('extension_one') do |ext|
@@ -391,15 +454,66 @@ describe Rake::ExtensionTask do
         end
       end
 
+      it "should set required_ruby_version from RUBY_CC_VERSION, set platform, clear extensions but keep metadata" do
+        platforms = ["x86-mingw32", "x64-mingw32"]
+        ruby_cc_versions = ["1.8.6", "2.1.10", "2.2.6", "2.3.3", "2.10.1"]
+        ENV["RUBY_CC_VERSION"] = ruby_cc_versions.join(":")
+        config = Hash.new
+        ruby_cc_versions.each do |ruby_cc_version|
+          platforms.each do |platform|
+            allow(config).to receive(:[]).
+              with("rbconfig-#{platform}-#{ruby_cc_version}").
+              and_return("/rubies/#{ruby_cc_version}/rbconfig.rb")
+          end
+        end
+        allow(YAML).to receive(:load_file).and_return(config)
+
+        allow(Gem).to receive_message_chain(:configuration, :verbose=).and_return(true)
+
+        spec = Gem::Specification.new do |s|
+          s.name = 'my_gem'
+          s.platform = Gem::Platform::RUBY
+          s.extensions = ['ext/somegem/extconf.rb']
+          s.metadata['allowed_push_host'] = 'http://test'
+        end
+
+        cross_specs = []
+        Rake::ExtensionTask.new("extension_one", spec) do |ext|
+          ext.cross_platform = platforms
+          ext.cross_compile = true
+          ext.cross_compiling do |cross_spec|
+            cross_specs << cross_spec
+          end
+        end
+        platforms.each do |platform|
+          Rake::Task["native:my_gem:#{platform}"].execute
+        end
+
+        expected_required_ruby_versions = [
+          Gem::Requirement.new([">= 1.8", "< 2.11"]),
+          Gem::Requirement.new([">= 1.8", "< 2.11"]),
+        ]
+        cross_specs.collect(&:required_ruby_version).should == expected_required_ruby_versions
+        cross_specs.collect(&:extensions).should == [[], []]
+        cross_specs.collect(&:platform).collect(&:to_s).should == platforms
+        cross_specs.collect{|s| s.metadata['allowed_push_host']}.should == ['http://test', 'http://test']
+
+        # original gemspec should keep unchanged
+        spec.required_ruby_version.should == Gem::Requirement.new([">= 0"])
+        spec.platform.should == Gem::Platform::RUBY
+        spec.extensions.should == ['ext/somegem/extconf.rb']
+        spec.metadata['allowed_push_host'].should == 'http://test'
+      end
+
       after :each do
         ENV.delete('RUBY_CC_VERSION')
       end
 
       context "(cross compile for multiple versions)" do
         before :each do
-          config = mock(Hash)
-          config.stub!(:[]).and_return('/rubies/1.8.6/rbconfig.rb', '/rubies/1.9.1/rbconfig.rb')
-          YAML.stub!(:load_file).and_return(config)
+          config = Hash.new
+          allow(config).to receive(:[]).and_return('/rubies/1.8.6/rbconfig.rb', '/rubies/1.9.1/rbconfig.rb')
+          allow(YAML).to receive(:load_file).and_return(config)
 
           ENV['RUBY_CC_VERSION'] = '1.8.6:1.9.1'
           @ext = Rake::ExtensionTask.new('extension_one') do |ext|
@@ -426,15 +540,9 @@ describe Rake::ExtensionTask do
           it 'should chain fake task to Makefile generation' do
             Rake::Task["tmp/universal-unknown/extension_one/#{@ruby_ver}/Makefile"].prerequisites.should include("tmp/universal-unknown/extension_one/#{@ruby_ver}/fake.rb")
           end
-        end
 
-        context 'rbconfig' do
-          it 'should chain rbconfig tasks to Makefile generation' do
-            Rake::Task["tmp/universal-unknown/extension_one/#{@ruby_ver}/Makefile"].prerequisites.should include("tmp/universal-unknown/extension_one/#{@ruby_ver}/rbconfig.rb")
-          end
-
-          it 'should take rbconfig from rake-compiler configuration' do
-            Rake::Task["tmp/universal-unknown/extension_one/#{@ruby_ver}/rbconfig.rb"].prerequisites.should include(@config_path)
+          it 'should chain rbconfig tasks to fake.rb generation' do
+            Rake::Task["tmp/universal-unknown/extension_one/#{@ruby_ver}/fake.rb"].prerequisites.should include(@config_path)
           end
         end
 
@@ -451,7 +559,7 @@ describe Rake::ExtensionTask do
 
         context 'compile:universal-unknown' do
           it "should be defined" do
-            Rake::Task.task_defined?('compile:universal-unknown').should be_true
+            Rake::Task.task_defined?('compile:universal-unknown').should == true
           end
 
           it "should depend on 'compile:extension_one:universal-unknown'" do
@@ -461,7 +569,7 @@ describe Rake::ExtensionTask do
 
         context 'native:universal-unknown' do
           it "should be defined" do
-            Rake::Task.task_defined?('native:universal-unknown').should be_true
+            Rake::Task.task_defined?('native:universal-unknown').should == true
           end
 
           it "should depend on 'native:my_gem:universal-unknown'" do
@@ -499,7 +607,7 @@ describe Rake::ExtensionTask do
   end
 
   def mock_gem_spec(stubs = {})
-    mock(Gem::Specification,
+    double(Gem::Specification,
       { :name => 'my_gem', :platform => 'ruby', :files => [] }.merge(stubs)
     )
   end
@@ -536,6 +644,6 @@ describe Rake::ExtensionTask do
   end
 
   def mock_fake_rb
-    mock(File, :write => 45)
+    double(File, :write => 45)
   end
 end
